/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.adapter;

import java.util.List;

import de.fzk.iai.ilcd.service.client.impl.vo.dataset.ContactDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.DataSetList;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.FlowDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.FlowPropertyDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.LCIAMethodDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.ProcessDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.SourceDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.dataset.UnitGroupDataSetVO;
import de.fzk.iai.ilcd.service.model.IContactVO;
import de.fzk.iai.ilcd.service.model.IDataSetListVO;
import de.fzk.iai.ilcd.service.model.IFlowPropertyVO;
import de.fzk.iai.ilcd.service.model.IFlowVO;
import de.fzk.iai.ilcd.service.model.ILCIAMethodVO;
import de.fzk.iai.ilcd.service.model.IProcessVO;
import de.fzk.iai.ilcd.service.model.ISourceVO;
import de.fzk.iai.ilcd.service.model.IUnitGroupVO;
import de.iai.ilcd.model.adapter.dataset.ContactAdapter;
import de.iai.ilcd.model.adapter.dataset.FlowAdapter;
import de.iai.ilcd.model.adapter.dataset.FlowPropertyAdapter;
import de.iai.ilcd.model.adapter.dataset.LCIAMethodAdapter;
import de.iai.ilcd.model.adapter.dataset.ProcessAdapter;
import de.iai.ilcd.model.adapter.dataset.SourceAdapter;
import de.iai.ilcd.model.adapter.dataset.UnitGroupAdapter;
import de.iai.ilcd.model.contact.Contact;
import de.iai.ilcd.model.flow.Flow;
import de.iai.ilcd.model.flowproperty.FlowProperty;
import de.iai.ilcd.model.lciamethod.LCIAMethod;
import de.iai.ilcd.model.source.Source;
import de.iai.ilcd.model.unitgroup.UnitGroup;

public class DataSetListAdapter extends DataSetList {

	public DataSetListAdapter(List<? extends IDataSetListVO> list) {

		try {
			for (IDataSetListVO dataset : list) {

				if (dataset instanceof de.iai.ilcd.model.process.Process) {
					this.getDataSet()
							.add(new ProcessAdapter(ProcessDataSetVO.class, (IProcessVO) dataset).getDataSet());
				} else if (dataset instanceof LCIAMethod) {
					this.getDataSet().add(
							new LCIAMethodAdapter(LCIAMethodDataSetVO.class, (ILCIAMethodVO) dataset).getDataSet());
				} else if (dataset instanceof Flow) {
					this.getDataSet().add(new FlowAdapter(FlowDataSetVO.class, (IFlowVO) dataset).getDataSet());
				} else if (dataset instanceof FlowProperty) {
					this.getDataSet().add(
							new FlowPropertyAdapter(FlowPropertyDataSetVO.class, (IFlowPropertyVO) dataset)
									.getDataSet());
				} else if (dataset instanceof UnitGroup) {
					this.getDataSet().add(
							new UnitGroupAdapter(UnitGroupDataSetVO.class, (IUnitGroupVO) dataset).getDataSet());
				} else if (dataset instanceof Source) {
					this.getDataSet().add(new SourceAdapter(SourceDataSetVO.class, (ISourceVO) dataset).getDataSet());
				} else if (dataset instanceof Contact) {
					this.getDataSet()
							.add(new ContactAdapter(ContactDataSetVO.class, (IContactVO) dataset).getDataSet());
				}
			}
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
