/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.adapter.dataset;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.fzk.iai.ilcd.service.client.impl.vo.dataset.DataSetVO;
import de.fzk.iai.ilcd.service.model.IDataSetListVO;
import de.fzk.iai.ilcd.service.model.IDataSetVO;
import de.fzk.iai.ilcd.service.model.common.ILString;
import de.iai.ilcd.model.adapter.ClassificationAdapter;

public abstract class AbstractDatasetAdapter<T extends DataSetVO, U extends IDataSetListVO> {

	private static final Logger logger = LoggerFactory.getLogger(AbstractDatasetAdapter.class);

	protected T dataset;

	public AbstractDatasetAdapter(Class<? extends T> adapterType, U adaptee) throws InstantiationException,
			IllegalAccessException {
		
		logger.trace("creating adapter from {} to {}", adaptee.getClass().getName(), adapterType.getName());

		this.dataset = adapterType.newInstance();
		
		this.dataset.setUuid(adaptee.getUuidAsString());
		this.dataset.setDataSetVersion(adaptee.getDataSetVersion());
		this.dataset.setPermanentUri(adaptee.getPermanentUri());

		// TODO copy all names, this doesn't work yet since adaptee.getName().getLStrings() is empty
		this.dataset.setName(adaptee.getName().getValue());
//		for (ILString l : adaptee.getName().getLStrings()) {
//			this.dataset.setName(l.getLang(), l.getValue());
//		}

		try {
			this.dataset.setClassification(new ClassificationAdapter(adaptee.getClassification()));
		} catch (Exception e) {
		}

		this.dataset.setSourceId(adaptee.getSourceId());
		this.dataset.setHref(adaptee.getHref());

		
		if (adaptee instanceof IDataSetVO) {
			logger.trace("{} is an instance of IDatSetVO", adaptee.getClass().getName());
			for (ILString l : ((IDataSetVO) adaptee).getDescription().getLStrings()) {
				this.dataset.setGeneralComment(l.getLang(), l.getValue());
			}
		}
	}

	public T getDataSet() {
		return dataset;
	}
}
