/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.adapter.dataset;

import de.fzk.iai.ilcd.service.client.impl.vo.dataset.FlowDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.types.common.LString;
import de.fzk.iai.ilcd.service.client.impl.vo.types.flow.ReferenceFlowPropertyType;
import de.fzk.iai.ilcd.service.model.IFlowListVO;
import de.fzk.iai.ilcd.service.model.IFlowVO;
import de.fzk.iai.ilcd.service.model.common.ILString;
import de.iai.ilcd.model.adapter.FlowCategorizationAdapter;
import de.iai.ilcd.model.adapter.GlobalReferenceAdapter;

public class FlowAdapter extends AbstractDatasetAdapter<FlowDataSetVO, IFlowListVO> {

	public FlowAdapter(Class<FlowDataSetVO> adapterClass, IFlowListVO adaptee) throws InstantiationException,
			IllegalAccessException {
		super(adapterClass, adaptee);

		try {
			this.dataset.setFlowCategorization(new FlowCategorizationAdapter(adaptee.getFlowCategorization()));
		} catch (Exception e) {
			// Ignore
		}

		ReferenceFlowPropertyType flowProperty = new ReferenceFlowPropertyType();
		flowProperty.setDefaultUnit(adaptee.getReferenceFlowProperty().getDefaultUnit());
		flowProperty.setHref(adaptee.getReferenceFlowProperty().getHref());

		for (ILString s : adaptee.getReferenceFlowProperty().getName().getLStrings()) {
			flowProperty.addName(new LString(s.getLang(), s.getValue()));
		}

		flowProperty.setReference(new GlobalReferenceAdapter(adaptee.getReferenceFlowProperty().getReference()));

		this.dataset.setReferenceFlowProperty(flowProperty);

		this.dataset.setType(adaptee.getType());

		if (adaptee instanceof IFlowVO) {
			for (ILString s : ((IFlowVO) adaptee).getSynonyms().getLStrings()) {
				this.dataset.getSynonyms().setValue(s.getLang(), s.getValue());
			}
			this.dataset.setCasNumber(((IFlowVO) adaptee).getCasNumber());
			this.dataset.setSumFormula(((IFlowVO) adaptee).getSumFormula());
		}
	}
}
