/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package de.iai.ilcd.model.adapter.dataset;

import de.fzk.iai.ilcd.service.client.impl.vo.dataset.ProcessDataSetVO;
import de.fzk.iai.ilcd.service.client.impl.vo.types.process.AccessInformationType;
import de.fzk.iai.ilcd.service.client.impl.vo.types.process.LCIMethodInformationType;
import de.fzk.iai.ilcd.service.client.impl.vo.types.process.TimeInformationType;
import de.fzk.iai.ilcd.service.model.IProcessListVO;
import de.fzk.iai.ilcd.service.model.IProcessVO;
import de.fzk.iai.ilcd.service.model.common.ILString;

public class ProcessAdapter extends AbstractDatasetAdapter<ProcessDataSetVO, IProcessListVO> {

	public ProcessAdapter(Class<ProcessDataSetVO> adapterClass, IProcessListVO adaptee) throws InstantiationException,
			IllegalAccessException {
		super(adapterClass, adaptee);

		AccessInformationType accessInformation = new AccessInformationType();
		for (ILString lstring : adaptee.getAccessInformation().getUseRestrictions().getLStrings()) {
			accessInformation.getUseRestrictions().setValue(lstring.getLang(), lstring.getValue());
		}
		accessInformation.setLicenseType(adaptee.getAccessInformation().getLicenseType());
		this.dataset.setAccessInformation(accessInformation);

		TimeInformationType timeInformation = new TimeInformationType();
		timeInformation.setReferenceYear(adaptee.getTimeInformation().getReferenceYear());
		timeInformation.setValidUntil(adaptee.getTimeInformation().getValidUntil());
		this.dataset.setTimeInformation(timeInformation);

		this.dataset.setType(adaptee.getType());

		LCIMethodInformationType lciMethodInformation = new LCIMethodInformationType();
		lciMethodInformation.setMethodPrinciple(adaptee.getLCIMethodInformation().getMethodPrinciple());
		lciMethodInformation.getApproaches().addAll(adaptee.getLCIMethodInformation().getApproaches());
		this.dataset.setLCIMethodInformation(lciMethodInformation);

		this.dataset.setLocation(adaptee.getLocation());

		this.dataset.setOverallQuality(adaptee.getOverallQuality());

		if (adaptee instanceof IProcessVO) {
			// adaptee.getApprovedBy()
			//
			// adaptee.getBaseName()
			//
			// adaptee.getCompletenessProductModel()
			//
			// adaptee.getComplianceSystems()
			//
			// adaptee.getFormat()
			//
			// adaptee.getOwnerReference()
			//
			// adaptee.getQuantitativeReference()
			//
			// adaptee.getReviews()
			//
			// adaptee.getSynonyms()
			//
			// adaptee.getTechnicalPurpose()
			//
			// adaptee.getUseAdvice()
		}

	}
}
