/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.security;

import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.subject.SimplePrincipalCollection;

import de.iai.ilcd.model.dao.UserDao;
import de.iai.ilcd.model.security.User;

/**
 * 
 * @author clemens.duepmeier
 */
public class IlcdAuthenticationInfo implements AuthenticationInfo {

	private AuthenticationToken authToken;

	private UserDao userDao = new UserDao();

	public IlcdAuthenticationInfo(AuthenticationToken at) {
		this.authToken = at;
	}

	@Override
	public PrincipalCollection getPrincipals() {
		PrincipalCollection principles = new SimplePrincipalCollection(getUser().getUserName(),
				IlcdSecurityRealm.REALM_NAME);
		return principles;
	}

	@Override
	public Object getCredentials() {
		return getUser().getPassword();
	}

	private User getUser() {
		User user = null;
		try {
			if (authToken == null)
				return null;
			user = userDao.getUser((String) authToken.getPrincipal());
		} catch (Exception e) {
			// we will return null in any case there is an error
		}
		return user;
	}

}
