/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.security;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.model.security.SecurityRole;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean(name = "user")
@RequestScoped
public class UserAccessBean {
	private static Logger logger = LoggerFactory.getLogger(UserAccessBean.class);

	public boolean isLoggedIn() {
		Subject currentUser = SecurityUtils.getSubject();
		if (currentUser == null)
			return false;

		if (currentUser.isAuthenticated()) {
			return true;
		} else
			return false;
	}

	public boolean isAdmin() {
		return hasRole("ADMIN");
	}

	public boolean isSuperAdmin() {
		String userName = this.getUserName();
		if (userName == null)
			return false;
		else if (userName.equals("root") && this.isLoggedIn()) {
			return true;
		} else
			return false;
	}

	public boolean isHasFullViewRights() {
		Subject currentUser = SecurityUtils.getSubject();
		// first check, if full view on data sets and data sets are public
		ConfigurationService confService = new ConfigurationService();
		boolean metaDataOnlyConfValue = confService.getProperties().getBoolean("security.guest.metadataOnly", true);
		if (!metaDataOnlyConfValue)
			return true; // everybody has full view rights on data sets
		// second, if not; verify that user has full view rights
		boolean userHasFullViewRights = this.hasRole("EXPORT"); // export means, you can download the whole data set;
																// i.e. view full data set
		if (userHasFullViewRights)
			logger.info("user {} has EXPORT rights", currentUser.getPrincipal());
		else {
			logger.info("user {} has no EXPORT rights", currentUser.getPrincipal());
		}
		return userHasFullViewRights;
	}

	public boolean hasRole(String role) {
		Subject currentUser = SecurityUtils.getSubject();
		if (currentUser == null) {
			logger.info("There is a security check on role {} but currently no user logged in", role);
			return false;
		}
		if (!currentUser.isAuthenticated()) {
			logger.info("There is a currentUser object but the current user is not authenticated");
			return false;
		}
		boolean hasRole = currentUser.hasRole(role);
		if (hasRole)
			logger.info("The user {} has role {}", currentUser.getPrincipal().toString(), role);
		else
			logger.info("The user {} does not has role {}", currentUser.getPrincipal().toString(), role);
		return hasRole;
	}

	public boolean hasRole(SecurityRole role) {
		return this.hasRole(role.getValue());
	}

	public String getUserName() {
		Subject currentUser = SecurityUtils.getSubject();
		if (currentUser == null)
			return null;
		else
			return (String) currentUser.getPrincipal();
	}

}
