/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.services;

import java.net.URI;
import java.net.URISyntaxException;

import javax.ws.rs.core.UriInfo;

import org.apache.commons.configuration.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.configuration.ConfigurationService;

/**
 * 
 * @author clemens.duepmeier
 */
public class ServiceContext {

	private Logger logger = LoggerFactory.getLogger(ServiceContext.class);

	// context coming from Jersey
	private UriInfo context;

	private ConfigurationService confService = new ConfigurationService();

	public ServiceContext(UriInfo serviceContext) {
		this.context = serviceContext;
	}

	public URI getBaseUri() {
		URI uri = context.getBaseUri();
		String hostName = uri.getHost();
		int port = uri.getPort();
		String path = uri.getPath();

		Configuration conf = confService.getProperties();
		String confHostName = null;
		if (conf.containsKey("service.url.hostname")) {
			confHostName = conf.getString("service.url.hostname");
		}
		if (conf.containsKey("service.url.port") && conf.getString("service.url.port") != null) {
			try {
				port = conf.getInt("service.url.port");
			} catch (Exception e) {
				// illegal service url port number in configuration file; ignore it
				logger.error("cannot parse port number in configuration file; will ignore it");
			}
		}
		// OK, if our hostName differs from configuration then we will use that (maybe a proxy)
		if (confHostName != null && !confHostName.equals(hostName)) {
			try {
				URI newUri = new URI("http", null, confHostName, port, path, null, null);
				uri = newUri;
			} catch (URISyntaxException e) {
				logger.error("Cannot create service base URL from configuration");
				// we ignore this because the built URI is not legal; so we use the standard URI from Jersey
			}
		}
		return uri;
	}

	public String getNodeId() {
		return ConfigurationService.getNodeId();
	}

	public String getNodeName() {
		return ConfigurationService.getNodeName();
	}
}
