/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.iai.ilcd.webgui.controller.admin;

import java.util.Properties;

import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.model.security.User;
import de.iai.ilcd.services.VelocityUtil;
import de.iai.ilcd.webgui.controller.ConfigurationBean;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.apache.commons.configuration.Configuration;
import org.apache.velocity.VelocityContext;

/**
 *
 * @author clemens.duepmeier
 */
public class RegistrationEmailGenerator {

    Configuration properties = null;
    ConfigurationBean confBean = null;
    public final String FALLBACK_SITENAME="KIT SODA4LCA test site";
    private String siteName=FALLBACK_SITENAME;
    
    public RegistrationEmailGenerator() {
        ConfigurationService confService = new ConfigurationService();
        properties = ConfigurationService.getProperties();
        confBean = new ConfigurationBean();
        
        siteName = properties.getString("mail.sitename", FALLBACK_SITENAME);

    }

    public Message genActivationMailForProvider(User user) throws MessagingException {
        // initialize message, should set from address and mailhost information
        Message message = getInitializedMessage();

        String providerRegistrationAddress = properties.getString("user.registration.registrationAddress");
        // can only be called if address is set right
        if (providerRegistrationAddress == null || providerRegistrationAddress.equals("")) {
            throw new MessagingException("Address for registration confirmation of provider is not set");
        }
        InternetAddress to;
        try {
            to = new InternetAddress(providerRegistrationAddress);
            message.setRecipient(Message.RecipientType.TO, to);
        } catch (MessagingException ex) {
            throw new MessagingException("Email address " + providerRegistrationAddress + " for registration confirmation seems to be wron", ex);
        }        

        // set subject of message
        message.setSubject("A new user " + user.getUserName() + " has registered at your " + siteName + ". Please check the email body for activation details");
        // set content of message
        VelocityContext context=this.getContext(user, siteName);
        setContentForActivationEmail(message, context, "/providerActivationMail.vm");

        return message;
    }

    public Message genActivationEmailForUser(User user) throws MessagingException {
        // initialize message, should set from address and mailhost information
        Message message = getInitializedMessage();

        // set to address
        InternetAddress to;
        try {
            to = new InternetAddress(user.getEmail());
            message.setRecipient(Message.RecipientType.TO, to);
        } catch (MessagingException ex) {
            throw new MessagingException("Is your email address " + user.getEmail() + " wrong? Cannot set your email address in registration email!", ex);
        }

        // set subject of message
        message.setSubject("Please confirm your user registration at our " + siteName);
        // set content of message
        VelocityContext context=this.getContext(user, siteName);
        setContentForActivationEmail(message, context, "/activationMail.vm");

        return message;
    }

    public Message genUserNotice(User user) throws MessagingException {
        // initialize message, should set from address and mailhost information
        Message message = getInitializedMessage();

        // set to address
        InternetAddress to;
        try {
            to = new InternetAddress(user.getEmail());
            message.setRecipient(Message.RecipientType.TO, to);
        } catch (MessagingException ex) {
            throw new MessagingException("Is your email address " + user.getEmail() + " wrong? Cannot set your email address in registration email!", ex);
        }

        // set subject of message
        message.setSubject("Your user registration at our " + siteName);
        // set content of message
        VelocityContext context=this.getContext(user, siteName);
        String providerRegistrationAddress = properties.getString("user.registration.registrationAddress");
        context.put("registrationAddress", providerRegistrationAddress);
        setContentForActivationEmail(message, context, "/registrationMail.vm");

        return message;
    }

    private Message getInitializedMessage() throws MessagingException {
        String sender = properties.getString("mail.sender", "clemens.duepmeier@kit.edu");
        String mailhost = properties.getString("mail.hostname", "mailhost.iai.fzk.de");

        Properties mailProps = new Properties();
        mailProps.put("mail.smtp.host", mailhost);
        Session session = Session.getDefaultInstance(mailProps);
        Message message = new MimeMessage(session);
        InternetAddress from = null;
        try {
            from = new InternetAddress(sender);
            message.setFrom(from);
        } catch (MessagingException ex) {
            MessagingException exNew = new MessagingException("Cannot initialize registration email message with sender and host information; Please contact the adminstration of the website", ex);
            throw exNew;
        }

        return message;
    }
    
    private VelocityContext getContext(User user, String siteName) {
        // we generate the message with a velocity template
        VelocityContext context = VelocityUtil.getContext();
        context.put("user", user);


        context.put("siteName", siteName);
        
        return context;
    }

    private void setContentForActivationEmail(Message message, VelocityContext context, String templateName) throws MessagingException {

        String activationUrl = confBean.getBaseUri().toString()
                + "/activation.xhtml";
        context.put("activationUrl", activationUrl);
        String text = VelocityUtil.parseTemplate(templateName, context);
        // now let's set the body of the message
        try {
            message.setContent(text, "text/html");
        } catch (MessagingException ex) {
            throw new MessagingException("Cannot set content of activation email", ex);
        }
    }
}
