/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/
package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Transport;

import org.apache.commons.configuration.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.configuration.ConfigurationService;
import de.iai.ilcd.model.common.GeographicalArea;
import de.iai.ilcd.model.dao.GeographicalAreaDao;
import de.iai.ilcd.model.dao.UserDao;
import de.iai.ilcd.model.security.Address;
import de.iai.ilcd.model.security.IndustrialSector;
import de.iai.ilcd.model.security.User;
import de.iai.ilcd.security.UserAccessBean;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean
@ViewScoped
public class UserHandler implements Serializable {

	private static final Logger logger = LoggerFactory.getLogger(UserDao.class);

	User user = new User();

	String verifyPassword = "";

	String savedPassword = null;

	int spamSliderValue = 0;

	int chosenSector = 0;

	List<IndustrialSector> industrialSectors = null;

	List<GeographicalArea> countries = null;

	int chosenCountry = 0;

	/** Creates a new instance of UserHandler */
	public UserHandler() {
		UserDao userDao = new UserDao();
		// initialize industrialSectors list
		industrialSectors = userDao.getIndustrialSectors();
		GeographicalAreaDao gDao = new GeographicalAreaDao();
		countries = gDao.getCountries();

		// check if the page was called with a request parameter specifying the current user
		String userId = FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get("userId");
		if (userId != null) {
			User existingUser = userDao.getUser(Long.parseLong(userId));
			if (existingUser != null) {
				user = existingUser;
				logger.info("found user with username {} and id {}", user.getUserName(), user.getId());
			}
		}
		// logger.error("View info of view is: {}", FacesContext.getCurrentInstance().getViewRoot().getViewId());
		String viewId = FacesContext.getCurrentInstance().getViewRoot().getViewId();
		if (viewId.contains("profile.xhtml")) {
			// OK, we are loaded by a profile page
			// first check if there is a valid user
			UserAccessBean callingUser = new UserAccessBean();
			if (!callingUser.isLoggedIn()) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
						"A user can change his profile only, if he is loggend in", "");
				FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
				return;
			}
			User currentUser = userDao.getUser(callingUser.getUserName());
			if (currentUser == null) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "The user with name "
						+ callingUser.getUserName() + " has no associated profile in the database", "");
				FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
				return;
			}
			user = currentUser;

		}
		// don't forget to save password, in case password field is empty
		savedPassword = user.getPassword();
		// also, don't forget to set index of chosen sector and country
		IndustrialSector sector = user.getSector();
		if (sector != null) {
			chosenSector = sector.getId().intValue();
		}
		if (user.getAddress() != null) {
			String countryCode = user.getAddress().getCountry();
			if (countryCode != null) {
				GeographicalArea country = gDao.getArea(countryCode);
				if (country != null) {
					chosenCountry = country.getId().intValue();
				}
			}
		} else {
			user.setAddress(new Address());
		}

	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public String getVerifyPassword() {
		return verifyPassword;
	}

	public void setVerifyPassword(String verifyPassword) {
		this.verifyPassword = verifyPassword;
	}

	public String getSavedPassword() {
		return savedPassword;
	}

	public void setSavedPassword(String savedPassword) {
		this.savedPassword = savedPassword;
	}

	public int getSpamSliderValue() {
		return spamSliderValue;
	}

	public void setSpamSliderValue(int spamSliderValue) {
		this.spamSliderValue = spamSliderValue;
	}

	public int getChosenSector() {
		return chosenSector;
	}

	public void setChosenSector(int chosenSector) {
		this.chosenSector = chosenSector;
	}

	public List<IndustrialSector> getIndustrialSectors() {
		return industrialSectors;
	}

	public int getChosenCountry() {
		return chosenCountry;
	}

	public void setChosenCountry(int chosenCountry) {
		this.chosenCountry = chosenCountry;
	}

	public List<GeographicalArea> getCountries() {
		return countries;
	}

	public void createUser() {
		UserDao dao = new UserDao();
		String statusMessage;

		UserAccessBean creatingUser = new UserAccessBean();
		if (!(creatingUser.hasRole("MANAGE_USER") || creatingUser.isSuperAdmin())) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"Only users with rights to manage user accounts can create new users", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		if (user.getUserName() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "There must be a unique user name!",
					"");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}
		if (user.getPassword() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "You need to supply a password", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}
		if (verifyPassword == null || !verifyPassword.equals(user.getPassword())) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"password and repeated password are not the same", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}
		User existingUser = dao.getUser(user.getUserName());
		if (existingUser != null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"A user with this user name already exists in the system", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

		// set sector and country information in user object
		setSectorAndCountry();

		try {
			dao.persist(user);
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "The user account has been created",
					"");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		} catch (Exception e) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"There were some erros while creating the user account; please try again", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

	}

	public String newUserRegistration() {
		Configuration conf = ConfigurationService.getProperties();

		UserDao dao = new UserDao();

		if (user.getUserName() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "There must be a unique user name!",
					"");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return null;
		}

		User existingUser = dao.getUser(user.getUserName());
		if (existingUser != null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"A user with this user name already exists in the system; choose another user name", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return null;
		}

		// set sector in country information in user object
		setSectorAndCountry();

		// generate password and activation key
		String password = RandomPassword.getPassword(8);
		user.setPassword(password);
		String registrationKey = RandomPassword.getPassword(20);
		user.setRegistrationKey(registrationKey);

		// generate Email messages
		Message userMessage = null;
		Message providerMessage = null;
		RegistrationEmailGenerator emailGenerator = new RegistrationEmailGenerator();
		boolean userSelfActivation = false;
		// get property, how user activation should be performed
		userSelfActivation = conf.getBoolean("user.registration.selfActivation", Boolean.FALSE);
		try {
			if (userSelfActivation) {
				// we send only an email to the user because he can activate his account himself
				userMessage = emailGenerator.genActivationEmailForUser(user);
			} else {
				// user only get's a notification about the completed registration
				userMessage = emailGenerator.genUserNotice(user);
				// provider get's an email to activate the user
				providerMessage = emailGenerator.genActivationMailForProvider(user);
			}
		} catch (MessagingException ex) {
			// OK, cannot generate email, let's show the error message to the user
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, ex.getMessage(), "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			logger.error(ex.getMessage(), ex);
			return null;
		}

		try {
			dao.persist(user);
			if (userMessage != null) {
				Transport.send(userMessage);
			}
			if (providerMessage != null) {
				Transport.send(providerMessage);
			}
		} catch (Exception ex) {
			dao.remove(user);
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"There was an unknown failure in the user registration process; please try again", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			logger.error("There was an error while saving registration information or sending registration email", ex);
			return null;
		}

		return "registrated";

	}

	private void setSectorAndCountry() {
		// now, set sector and country
		IndustrialSector sector = null;
		for (IndustrialSector s : industrialSectors) {
			if (s.getId().intValue() == chosenSector) {
				sector = s;
			}
		}
		if (sector != null) {
			user.setSector(sector);
		}
		GeographicalArea country = null;
		for (GeographicalArea c : countries) {
			if (c.getId().intValue() == chosenCountry) {
				country = c;
			}
		}
		if (country != null) {
			user.getAddress().setCountry(country.getAreaCode());
		}
	}

	public void changeUser() {
		UserDao dao = new UserDao();
		String statusMessage;

		UserAccessBean changingUser = new UserAccessBean();
		if (changingUser == null || !changingUser.isLoggedIn()) {
			FacesMessage facesMsg = new FacesMessage(
					FacesMessage.SEVERITY_WARN,
					"Error: No user logged in! Only the user itself or a user with user management rights can change user entries",
					"");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}
		if (!(changingUser.isSuperAdmin() || changingUser.hasRole("MANAGE_USER") || changingUser.getUserName().equals(
				user.getUserName()))) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"Only the user itself or a user with user management rights can change user entries", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		if (user.getUserName() == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "There must be a unique user name!",
					"");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}
		if (user.getPassword() == null || user.getPassword().equals("")) {
			// OK, we don't change the password
			user.setPassword(savedPassword);
		} else if (verifyPassword == null || !verifyPassword.equals(user.getPassword())) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"password and repeated password are not the same", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		// set sector and country information in user object
		setSectorAndCountry();

		User existingUser = dao.getUser(user.getUserName());
		if (existingUser == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN, "A user with the user name "
					+ user.getUserName() + " does not exist", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
			return;
		}

		logger.info("Merge on user {} with user id {}", user.getUserName(), user.getId());

		try {
			dao.merge(user);
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "The user account has been changed",
					"");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		} catch (Exception e) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"There was an error while saving the data; please try again", "");
			FacesContext.getCurrentInstance().addMessage("errorInfo", facesMsg);
		}

	}
}
