/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.xml.read;

import java.io.PrintWriter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.dao.ContactDao;
import de.iai.ilcd.model.dao.DataSetDao;
import de.iai.ilcd.model.dao.FlowDao;
import de.iai.ilcd.model.dao.FlowPropertyDao;
import de.iai.ilcd.model.dao.LCIAMethodDao;
import de.iai.ilcd.model.dao.PersistType;
import de.iai.ilcd.model.dao.ProcessDao;
import de.iai.ilcd.model.dao.SourceDao;
import de.iai.ilcd.model.dao.UnitGroupDao;
import de.iai.ilcd.persistence.PersistenceUtil;
import de.schlichtherle.io.File;

/**
 * 
 * @author clemens.duepmeier
 */
public class DataSetZipImporter {
	private static final Logger logger = LoggerFactory.getLogger(DataSetZipImporter.class);

	public static void main(String[] args) {
		// TODO code application logic here

		DataSetZipImporter importer = new DataSetZipImporter();

		// String baseDirectory = "D:\\webapps\\apache-tomcat-5.5.12\\webapps\\sirimlca\\datasets\\ilcd\\";
		String zipFileName = "D:\\netzwerk\\ILCD.zip";

		PrintWriter out = new PrintWriter(System.out, true);
		importer.importZipFile(zipFileName, out);

	}

	public void importZipFile(String zipFileName, PrintWriter out) {

		logger.debug("importing ZIP archive " + zipFileName);

		File zipFile = new File(zipFileName);

		if (!zipFile.isArchive()) {
			out.println("Error: file " + zipFileName + " is not a zip archive!");
			out.println("Please upload only ILCD conformant zip archives.");
			out.flush();
		}

		String[] topDirectories = zipFile.list();
		// we try to parse to versions of zip files
		// one according to the ILCD zip file format; data sets should be in an ILCD subdirectory then
		// otherwise we assume that the data set directories are directly under the root of the zip file
		String subDir = ""; // means data sets are under the root directory
		for (String directory : topDirectories) {
			if (directory.equals("ILCD") || directory.equals("ilcd")) {
				// we have a normal ILCD archive with subdirectory ilcd hopefully containing the data sets
				// so set subdirectory to the data directory
				subDir = "/" + directory;
				break;
			}
		}

		ContactReader contactReader = new ContactReader();
		ContactDao contactDao = new ContactDao();
		File contactsDirectory = new File(zipFileName + subDir + "/contacts");
		if (!contactsDirectory.exists() || !contactsDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any contact data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing contact data sets");
				out.flush();
			}

			this.importFiles(contactsDirectory, contactReader, contactDao, out);
		}

		SourceReader sourceReader = new SourceReader();
		SourceDao sourceDao = new SourceDao();
		File sourcesDirectory = new File(zipFileName + subDir + "/sources");
		if (!sourcesDirectory.exists() || !sourcesDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any source data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing source data sets");
				out.flush();
			}

			this.importFiles(sourcesDirectory, sourceReader, sourceDao, out);
		}

		UnitGroupReader unitGroupReader = new UnitGroupReader();
		UnitGroupDao unitGroupDao = new UnitGroupDao();
		File unitGroupsDirectory = new File(zipFileName + subDir + "/unitgroups");
		if (!unitGroupsDirectory.exists() || !unitGroupsDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any unit group data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing unit group data sets");
				out.flush();
			}

			this.importFiles(unitGroupsDirectory, unitGroupReader, unitGroupDao, out);
		}

		FlowPropertyReader flowPropertyReader = new FlowPropertyReader();
		FlowPropertyDao flowPropertyDao = new FlowPropertyDao();
		File flowPropertiesDirectory = new File(zipFileName + subDir + "/flowproperties");
		if (!flowPropertiesDirectory.exists() || !flowPropertiesDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any flow property data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing flow property data sets");
				out.flush();
			}

			this.importFiles(flowPropertiesDirectory, flowPropertyReader, flowPropertyDao, out);
		}

		FlowReader flowReader = new FlowReader();
		FlowDao flowDao = new FlowDao();
		File flowsDirectory = new File(zipFileName + subDir + "/flows");
		if (!flowsDirectory.exists() || !flowsDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any flow data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing flow data sets");
				out.flush();
			}

			this.importFiles(flowsDirectory, flowReader, flowDao, out);
		}

		ProcessReader processReader = new ProcessReader();
		ProcessDao processDao = new ProcessDao();
		File processesDirectory = new File(zipFileName + subDir + "/processes");
		if (!processesDirectory.exists() || !processesDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any process data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing process data sets");
				out.flush();
			}
			this.importFiles(processesDirectory, processReader, processDao, out);
		}

		LCIAMethodReader lciamethodReader = new LCIAMethodReader();
		LCIAMethodDao lciamethodDao = new LCIAMethodDao();
		File lciamethodsDirectory = new File(zipFileName + subDir + "/lciamethods");
		if (!lciamethodsDirectory.exists() || !lciamethodsDirectory.isDirectory()) {
			if (out != null) {
				out.println("Notice: the zip file does not contain any lciamethod data sets");
				out.flush();
			}
		} else {
			if (out != null) {
				out.println("processing lciamethod data sets");
				out.flush();
			}
			this.importFiles(lciamethodsDirectory, lciamethodReader, lciamethodDao, out);
		}

		// flush output at end;
		if (out != null)
			out.flush();

		// don't forget to unmount zipFile and release resources of it
		try {
			File.umount(true);
		} catch (Exception e) {
			// do nothing, we just close all zip archive file resources
			logger.error("error unmounting zipfile", e);
		}

	}

	public void importFiles(File zipDirectory, DataSetReader reader, DataSetDao dao, PrintWriter out) {
		File[] files = null;

		int filesImported = 0;
		int filesNotImported = 0;

		try {
			files = (File[]) zipDirectory.listFiles();

			if (out != null) {
				logger.debug("importing from " + zipDirectory.getAbsolutePath());
				out.println("Number of files to import: " + files.length);
				out.flush();
			}

			for (File file : files) {
				boolean persisted = importFile(file, reader, dao, out);
				if (out != null) {
					if (persisted) {
						out.println("Imported file: " + file.getName());
						filesImported++;
					} else {
						out.println("FILE WAS NOT IMPORTED: " + file.getName());
						filesNotImported++;
					}
					out.flush();
				}
			}
		} catch (Exception e) {
			logger.error("Cannot import all files from directory: {}", zipDirectory.getAbsolutePath());
			logger.error("Exception is: ", e);
		}
		if (filesImported > 0) {
			out.println(filesImported + " files were imported");
			out.flush();
		}
		if (filesNotImported > 0) {
			out.println(filesNotImported + " files were NOT imported");
			out.flush();
		}
	}

	public boolean importFile(File dataSetFile, DataSetReader reader, DataSetDao<DataSet, ?> dao, PrintWriter out) {

		logger.debug("importing " + dataSetFile.getName());

		DataSet dataset;
		try {
			dataset = reader.readDataSetFromFile(dataSetFile, out);
		} catch (Exception ex) {
			logger.error("The file {} was not imported", dataSetFile.getAbsolutePath());
			logger.error("The exception is: ", ex);
			return false;
		}

		boolean persisted = dao.checkAndPersist(dataset, PersistType.ONLYNEW, out);

		PersistenceUtil.closeEntityManager();

		return persisted;
	}
}
